


--[[
-- TrueBld3 simpler version that uses less user created telemetry objects
--
-- ToDo

-- ?

-- https://www.gnu.org/licenses/gpl-3.0.en.html
-- OpenTX Lua script
-- TELEMETRY

 Created: 10/01/2018 by TrueBuild
 This script has been converted from a script I created 21/01/2015 for OpenTx 2.0.20
 It involved a lot more work than I anticipated ;)
 
 
 
 It was gratifying see a reference to my old script by Dean Church,
	who used my old version as a source/inspiration for his OpenTx 2.1.9 version.

 Thanks go to Dean Church for the user lipo capacity idea 
 
 The original version contained thanks to these guys.
 
    -- Some neat coding style ideas plagiarized from Dean Church
	-- Many ideas plagiarized from many other sources including NigelSheffield
	-- None of this would have been possible without the Debug output added to CompanionX by dinamich
	-- And to end, none of this would have worked without help from on4mh (Mike) on the 
	-- 		rcgroups LUA scripting - Technical discussion forum.
 
 
  Purpose of the script:-

  This is an attempt to get a realistic percentage remaining of the Lipo.
	The code uses the lowest resting cell voltage to get the mAhr remaining at the start of the flight.
		This also is a useful check for a full lipo; nothing worse than installing an exhausted lipo
	Once the throttle release is activated it subtracts the telemetry mAhr being used from the initial remaining mAhr.
	The Lipo percentage used is monitored, and used to trigger a warning sound at intervals of 10%.

 This script assumes ThrottleOnOff is the SD switch that has three positions so anything smaller than Zero is throttle hold
	if you use another switch look for :- local IdSwitch_D = getFieldInfo('sd').id  and change the sd to your switch.

  
  ]]--

--  This script needs to be placed in SCRIPTS/TELEMETRY directory on your transmitter
--     and the accompanying wav files need to be placed in SCRIPTS/TELEMETRY/TrueBld3 directory on your transmitter
	
-----------------------------------------------------------------
-- The Pilot can change specific settings here.
-----------------------------------------------------------------	
-- 	You must use discover sensors for this scrip to work.
-- 	and you must add the additional sensors as described in the relevant sensor section.
	
	---------------------------------------------------------------------
	-- Notes on the Sensors and how to create them
	-- Sensors needed by the system that must be created by the pilot! --
	---------------------------------------------------------------------
	
	-- 	You must use discover sensors first for this scrip to work.
	--  Do a search on discover sensors -- eg https://www.youtube.com/watch?v=J1qd5VfPRP4
	--  Note:- it is recommended that you tick the option box to log all these sensors to logfile.
	--      Log option is at the end of each sensor setup screen.
	
	-- The Sensor names are in quotes e.g. "Cels" (use the name without the quotes!)
	--		Sensor names please follow the same case usage!
	-- 		FLVSS Smart Port LiPo Voltage Sensor or 
	--		Mlvss Mini Lipo Voltage Sensor Smart port Enable without OLED Screen (default channel is 2)
	-- "Cels" -- Voltage sensor only does 1 to 6 cells 
	--
	-- 		Note : Only if you are using two voltage sensors.
	--		add facility for another voltage sensor for the 10 to 12 cell systems (48v)
	--			or even if you have just have a 24volt system using two 3 cell lipos in series,
	--			as do the Precision Aerobatic planes. 
	-- 		"Cel2" -- the second voltage sensor (it must have a unique ID / use a Servo Channel Changer)
	--
	--		FCS-150A smart port current sensor or FCS-40A smart port current sensor
	-- "Cons" -- this is a calculated sensor based on the Curr sensor "Curr" -- default name for current sensor
	--    Use the method described below to create the above sensor in the telemetry screen...
			-- The method is :- manually Add a new sensor and name it "Cons" 
			-- then change the drop-down from Custom to Calculated 
			--   then select Consumption called "Consumpt"
			--   then use the source sensor drop-down to select Curr. Job done.	
			
			
	-- new method means we do not need 	extra user created sensors	
	-- I'm going to keep some for logging and monitoring.
	--    You can do the same if you are interest in logging as shown below ;)	

	---------------------------------------------------------------------
	-- END of Notes on the Sensors and how to create them
	-- Sensors needed by the system that must be created by the pilot! --
	---------------------------------------------------------------------
	-- *************************************************************** --
	---------------------------------------------------------------------
  
  	
	---------------------------------------------------------------------
		-- Sensors that the pilot may find useful so he/she can log them! --
	---------------------------------------------------------------------		
	-- "CMin" method below...
	-- in the telemetry screen (manually Add a new sensor and name it "CMin"
		-- then change the drop-down from Custom to Calculated
		--   then use the sensor drop-down to select Cels , a drop-down appears, select Lowest. Job done.
	-- "CMax" creation method below...
		-- do the same as CMin as above to create this sensor and select Highest.
  
  	-- 		Note : Only if you are using two voltage sensors.
	
	-- 		"CMi2" -- short name for CMin number 2
	--		in the telemetry screen (manually Add a new sensor and name it "CMin"
			-- then change the drop-down from Custom to Calculated
			-- then use the sensor drop-down to select Cel2 , a drop-down appears, select Lowest. Job done.
	
	-- 		"CMa2" -- short name for CMax number 2
	--		in the telemetry screen (manually Add a new sensor and name it "CMin"
			-- then change the drop-down from Custom to Calculated
			-- then use the sensor drop-down to select Cel2 , a drop-down appears, select Lowest. Job done.
	---------------------------------------------------------------------

	--  End of  Sensors that the pilot may find useful so he/she can log them! -- --
	---------------------------------------------------------------------
	-- *************************************************************** --
	---------------------------------------------------------------------
  
  
  -- we are calculating Delta because we may need the Delta of two lipos
  

  -- Play Sound of Percentage used on Demand Switch
  -- my transmitter uses the SF switch and default the position is away from pilot
  -- sf                       Switch F
  -- if you do not want the option to repeat percentage remaining the just change the line below to...
  -- local SwitchF = ""
  local SwitchF = "sf"
  -- in other words remove sf
  


	-- a switch towards a user is 1024 -
	--          in the middle is 0
	-- 			and away from user is -1024 (my default that is has a warning if not)
	  
  
  -- Field name               Field description
  -- sd  					 Switch D
  local IdSwitch_D = getFieldInfo('sd').id   

  -- this value is updated by the system using GV variables from flight modes
  local LipoCapacity_mAh = 40  -- set a default if you only have one plane using this eg. 4000
  

  ---###############################################----
  --- (Dean Church's method setting lipo capacity)
  ------------------------------------------------------
  
 -- Do not change the next line
local GV = {[1] = 0, [2] = 1, [3] = 2,[4] = 3,[5] = 4,[6] = 5, [7] = 6, [8] = 7, [9] = 8}
----Do not change the above line --------------------------------------------------------
-- OpenTX Global Variables (GV)
--	These are global to the model and not between models.
--
--	Each flight mode (FM) has its own set of GVs. Using this script you could
--		be flying in FM 0 but access variables from FM 8. This is useful when
--		when running out of GVs available to use.
--		Most users can leave the flight mode setting at the default value.
--
-- For this system you must have a mAh Sensor
--
--- 	GVBatCap - Battery capacity provided as mAh/100,
--									2800 mAh would be 28, 800 mAh would be 8
--      If you are flying a model with a batch of 4000mAh = 40 batteries you reduce
--			 the value to compensate for the loss of capacity due to ageing,
-- 				e.g. 3800mAh = 38  


--
-- Change as desired
-- Use GV[6] for GV6, GV[7] for GV7 and so on
local GVBatCap = GV[7] 	-- Read Battery Capacity, 8 for 800mAh, 22 for 2200mAh
												-- The corresponding must be set under the FLIGHT MODES
												-- screen on the Tx.
												-- If the GV is 0 or not set on the Tx then
												-- % remaining is calculated based on battery voltage
												-- which may not be as accurate.
												
												-- This setting is very useful for a set of ageing lipo packs,
												--  by just setting a 2200mAh to a 2000mAh e.g. 20 or less.
												
local GVFlightMode = 8 -- Use a different flight mode if running out of GVs

 
 -----------------------------------------------------
  --- (End of Dean Church's method)
---###############################################----

 
 
 
 
------------------------------------------------- 
  -- logics for function PlaySoundFileOnDemand
-------------------------------------------------
local lPlayedUp = true --  have we played the sound 
local lPlayedDown = false
local lWeAreUsingSwitchF = true
local IdPlayOnDemandSwitch = 0 
local isFswitchUp = false
local lFirstTime = true
 ---------------------------------------------
  
  

  
  -- state the name of the script
  local myTitle = "Lipo Percentage Remaining Monitor"
  --local myTitle = "TrueBld3 Percentage Remaining Monitor"
  
  local x, y


  	----------------------------------------------------------------------
	-- Telemetry stuff
	----------------------------------------------------------------------
  -- You must use discover sensors for this scrip to work.
	
  -- voltage sensors are Cels by default or user named
  -- for now use Cels
  -- we are going to have to calculate this 
  local sCelsSensor = "Cels" -- Voltage sensor only does 1 to 6 cells 
  -- add facility for another voltage sensor for the 10 to 12 cell systems (48v)
  local sCelsSensor2 = "Cel2"
  local twoVotSensors = false
  
  -- consumption              Current sensor - consumption [mili amper hours]
  local sMillAmpHourSensor = "Cons" -- this is a calculated sensor based on the Curr sensor "Curr" -- default name for current sensor
  -- in the telemetry screen (manually Add a new sensor and name it "Cons" then change the drop-down from Custom to Calculated
  --   then use the sensor drop-down to select Curr. Job done.
 	
  -- if the user has added another (FLVSS Smart Port LiPo Voltage Sensor)
  -- or Mlvss Mini Lipo Voltage Sensor Smart port Enable without OLED Screen (default channel is 2)
  -------------------------------------------------------------------------------------
  -- keep this for testing ?
   -------------------------------------------------------------------------------------
  local sCMinVotsSensor2  = "CMi2"
  local sCMaxVotsSensor2  = "CMa2"
	 -------------------------------------------------------------------------------------
	-------------------------------------------------------------------------------------
  local sRSSI = "RSSI" -- default name for sensor RSSI in db
  local RSSIThreshold = 0
    
  local haveWeTwoVoltageSensors = false --
  local missingSensorsCount = 0

  
 	----------------------------------------------------------------------
	-- Receiver/Motor battery stuff -- we are really looking at the lipo with sensors
	----------------------------------------------------------------------

	local minCellVolts1 = 0.00 -- using a better name.
    local maxCellVolts1 = 0.00
	
	local minCellVolts2 = 0.00 -- using a better name.
    local maxCellVolts2 = 0.00
	
	local maxCellVolts = 0
	local minCellVolts = 0
	
	
	local minCommonCellVolts = 0 -- we need a common min cell voltage because we may have two voltage sensors.
  
  
  local lipo_mAh_AtFlightStart = 0
  local lipo_mAh_Remaining = 0
    
  local estimatedPercentRem = 0 -- value found in the InMyArray and calculated value
  local percentThatPlayedFile = 0 --  lipoLastPercentRem = 0 -- keep a record of last percent that fired a playFile()
  local estimatedPercentRem = 0 -- estimated lipo percent when the lipo is connected.
  
  --- and  this will come from my InMyArray function at the start
   -- Lipo percent at start of flight when transmitter is switched on
  --local lipoPercentRemStart = 0 -- this will come from my InMyArray function
local calcPercentRem = 0
  
  --------------------------------- 
  local deltaVolts= 0.00
  local warnIfDelta = .36  -- warn if (e.g. 4.14v - 3.72v)
  local lipoWarningDone = false -- stop this thing from saying over and over. Once a flight should do.
  

  local mAhs_Consumed = 0
	----------------------------------------------------------------------
	-- Transmitter battery stuff
	----------------------------------------------------------------------
	local TX_VoltageID = getFieldInfo('tx-voltage').id -- get the ID of the Tx Voltage using a search on the string

  local TX_Voltage = 0.00 -- the actual TX battery voltage
  local txPercent = 0
  local txMinVolts = 6.90 -- 80% used so call that all used

  -- local txMaxVolts = 8.3 TX is a six cell pack therefore that equals 1.383 per cell

  local txMaxVolts = 7.6 -- 8.3 was too high leaving a charged TX for about 16 hours it dropped to 8v
  -- So I'm unsure what the correct voltage should be to give a fair idea of capacity percentage
  -- found a graph that shows an unused fully charged cell stays at 1.25v (ish) for 30 days
  -- so I'm going to try 7.6 (that may still too high)
  




	----------------------------------------------------------------------
	-- 
	----------------------------------------------------------------------
 
  -- Some neat coding style ideas plagiarized from Dean Church
  -- x,y = column,row
  -- Position (0,0) is at top left.
  -- Position (63,212) is bottom right?

  local moveDownNLines = 0

-- using an array, values have been found by experiment (sourced from the web)
-- and are based on Hoppy and jj604's comments 

-- these values are only used to determine the state of lipo at the start of the flight!
local myArrayPercentList = {
{3.625,5},
{3.670, 10},
{3.715, 15},
{3.720, 20},
{3.730, 25},
{3.750, 30},
{3.770, 35},
{3.800, 40},
{3.826, 45},
{3.839, 50},
{3.861, 55},
{3.883, 60},
{3.910, 65},
{3.936, 70},
{3.986, 75},
{3.999, 80},
{4.042, 85},
{4.085, 90},
{4.142, 95},
{4.170, 97},
{4.200, 100}
}


local myArraySoundFiles = {[5] = "5Left.wav",[10] = "10Left.wav",[20] = "20Left.wav",[30] = "Bt30LftS.wav"
						,[40] = "Bt40LftS.wav",[50] = "Bt50LftS.wav",[60] = "Bt60LftS.wav"
						,[70] = "Bt70LftS.wav",[80] = "Bt80LftS.wav",[90] = "Bt90LftS.wav"}


						
local myArraySensors = {
    [1] = "Cels",
    [2] = "Cons",
  	[3] = "Curr"
	}
-- I think we should trap for Curr as well.
-- Cons = Calculated Consumption Sensor: Curr  and tick the Logs box


	
  local listOfMissingSensors = ""
  local missingSensorsWarning = ""
  local myArrayMissing = {}
  

-- to test	
--[[						
local myArraySensors = {
    [1] = "Cels",
    [2] = "Cons",
    [3] = "CMin",
    [4] = "CMax",
	[5] = "CDel",
	[6] = "Miss",
    [7] = "test",
    [8] = "anot",
}
 ]]--	

	
								

local fullSoundString = ""
-- put sound files in directory to match the script
local soundDirString = "/SCRIPTS/TELEMETRY/TrueBld3/"


-- let's have a logic so that it is clear whether the throttle is On or Off
local logicThrottleHold = true  -- we will use this to reset stuff / like playing the sound file
-- do the same for RSSI


local lCorrectVersionOpenTX = false


 local function	myRound(num,	decimals)
	-- named myRound so we know it is not a built in function!
	-- cannot remember where I got this from; so no credit given ;(
	
	-- we are in trouble if 0 is passed as num
	if num == nil then
		num = 0
	end
	
	local	mult	=	10^(decimals	or	0)
		--print("mult = " ,mult)
		--print("num = ",num)
	return	math.floor(num	*	mult	+	0.5)	/	mult 
 end

local function CountElements(t1)
	local nCounter = 0
	for i=1,#t1 do
 	   nCounter = nCounter + 1
    end
	return nCounter
end



local function ConcatString( t1 )
	local tmpString = ""
	for i=1, #t1 do
		-- Thanks Noircogi for the idea behind stopping the extra comma at the end of the string.
		-- Thanks to on4mj Mike for showing me how to implement it.
		-- before the guys showed me the above I used a CountElements function.
		if i == 1 then	-- that's the same as: tmpString <> ""
			tmpString = t1[ 1 ]
		else
			tmpString = tmpString .. ", " .. t1[ i ]
		end
	end
	return tmpString
end


local function GetIsFswitchUp()
	if lWeAreUsingSwitchF
	then
		-- up is away from pilot / the default setting of the switch
		-- so on the transmitter this is down and away from the pilot!
		-- it can be confusing but it matches the companion way of looking at switches where away is up for SF and SH
		-- So switch away/up value is -1024
		if getValue(IdPlayOnDemandSwitch) < 1
		then
			return true		
		else
			return false
		end
		
	else
		-- user is not using a switch
	end
 end

 
 local function playCurrentSoundFile()
	-- on demand play the last percentage file of the amount of lipo left.
	-- The idea being that the pilot can request here the amount of lipo left.
	-- For example the pilot may have been distracted when the last automatic sound file was played.
	
	
	if percentThatPlayedFile > 0 and not fullSoundString == ""
		then
			playFile(fullSoundString)
	
	end
	
end
 


local	function getTelemetryId(name)

	local field	= getFieldInfo(name)
	if	field	
	then	
		return	field.id	
	else					
		return	-1				
	
	end
end


--			default censors eg;- Cels, Curr and Cons because they will exist
--				while debugging. 


local function findPercentRemaining( myArrayPercentList, targetVoltage )
	local returnPercent = 10
	
	if targetVoltage > 4.200 or targetVoltage < 3.683
	then
			-- trap for odd values not in array
			if  targetVoltage > 4.200
			  then
				result = 100
			end 
			if  targetVoltage < 3.683
			  then
				 result = 10
			end
	else
			-- method of finding percent in my array provided by on4mh (Mike)
		  for i, v in ipairs( myArrayPercentList ) 
		  do
			if v[ 1 ] >= targetVoltage
			then
				returnPercent =  v[ 2 ] 
				break
			end
		  end
		
	end
	return returnPercent
end


local function selectAndPlaySound(myParameter)
	-- need now to run sound file to say what percentage left is.
	-- play sound if Modulus 10 = 0 (play sound only when battery 90,80,...)
	-- and we have not already played that percent sound

	local myModVal = 0
	print("selectAndPlaySound ", myParameter)
	print("selectAndPlaySound ", logicThrottleHold)
	if logicThrottleHold then
			-- do not play battery percentages when throttle is locked.
					
			
		else
			if estimatedPercentRem < 10 then
				-- we are not interested in values lower than 10
					estimatedPercentRem = 5
					myModVal = 0
				else
					-- mod to make all percentages in 10s, 20, 30  
					myModVal = estimatedPercentRem % 10
			end



			if myModVal == 0 and estimatedPercentRem ~= percentThatPlayedFile 
				then
					
					--fullSoundString = ""
					mySoundFileString = (myArraySoundFiles[estimatedPercentRem])
					-- need a trap for nil value returned from myArraySoundFiles
					if mySoundFileString == nil
						then
						-- print("no file found")
						else
						--playFile("SOUNDS/en/5Left.wav")
						fullSoundString = soundDirString .. mySoundFileString
						playFile(fullSoundString)
						-- do not keep playing the same sound file over and over
						percentThatPlayedFile = estimatedPercentRem
					end
				else
					if myParameter then
						-- play this sound file on demand 
						-- The pilot may have missed the sound or been distracted.
						-- The last percentage will be played again 
	
						-- need a trap for nil value returned from myArraySoundFiles
						if mySoundFileString == nil
							then
							-- print("no file found")
							else
							--playFile("SOUNDS/en/5Left.wav")
							--fullSoundString = soundDirString .. mySoundFileString
							playFile(fullSoundString)
							-- do not keep playing the same sound file over and over
							percentThatPlayedFile = estimatedPercentRem
						end
					
					
				
					end
				
				
				
			end
		
	end	
	

end



local function PlaySoundFileOnDemand()
	-- trap for switch toggling
	-- and stop the sound the first time the script is run / at model load
	-- and stop the sound from repeating until the switch is flipped again
	
	if lWeAreUsingSwitchF
	then
	
	    if GetIsFswitchUp() 
		then
			-- switch F up
			--logicF = true		
			if lPlayedUp 
			then
				-- do not play again until switch is flipped
				lPlayedDown = false
			else
				if lFirstTime
					then
						-- do nothing
					--	lPlayedDown = false
						lFirstTime = false
					else
	
						--playFile("/SCRIPTS/TELEMETRY/telNw2/Bt40LftS.wav")
						selectAndPlaySound(true)
					--playCurrentSoundFile()
						lPlayedUp = true
				end
			end				
		else
				-- switch F down		
			if lPlayedDown 
			then
				-- do not play again
				--lPlayedDown = true
				lPlayedUp = false
			else		
					
					--playFile("/SCRIPTS/TELEMETRY/telNw2/5Left.wav")
					selectAndPlaySound(true)
					--playCurrentSoundFile()
					lPlayedDown = true
				
			end
		end
	else
		-- user is not using a switch
	end

end

local function centreText(sString,myRow)

local screenLen = 46
local StringLength = string.len(sString)
local totSpace = screenLen - StringLength 
local column = totSpace/2
local addjusted = column * 5
addjusted = math.ceil(addjusted)
lcd.drawText(addjusted,myRow,sString,SMLSIZE)
	return 

end


local function correctVersionOpenTX()
  
	  local ver, radio, maj, minor, rev = getVersion()
	-- to test 
	--ver = "2.1.7"

	if ver == "2.1.9" then
		return true
	else
		 print("This script/program only tested with OpenTX 2.1.9")
		return false
	end
	
end 


local function returnTheLarger()
  local largeVal = RangeMax(10,20)
	--print(largeVal)
end


local function getFieldId(tSensor)
local	fieldinfo	=	getFieldInfo(tSensor)	
	local id = 0
		if	fieldinfo	then	
			id = fieldinfo['id']	
		else				
	
		end 
	return id
end


local function higherOfTwo(x,y)
	if x > y then
		return x
	else
		return y
	end
end 
local function lowerOfTwo(x,y)
	if x > y then
		return y
	else
		return x
	end
end 




local function findMinValue(telemetryObjectName)
	
	local minVoltage = 99
	cellId = getTelemetryId(telemetryObjectName)
	if cellId == -1 then
		-- telemetry object not found
		--print("telemetry object not found")
	else
		-- check we have table
		cellResult = getValue(cellId)
		if (type(cellResult) == "table") then
			--print("its a table")
			--Value = ""
			-- index, value
			--a = {4.1, 4.011, 3.720,3.9}
			--a = {8.1, 8.011, 7.5,7.9}
			for i, voltage in ipairs(cellResult) do
			--for i, voltage in ipairs(a) do
				--4.2
				if minVoltage > voltage then
					minVoltage = voltage
					--print("minVoltage = " .. minVoltage)
	
				else
					
				end 
			end
		else

		end
	end		
	if minVoltage == 99 then
		minVoltage = -1
	end
	
	return minVoltage 
end
local function findMaxValue(telemetryObjectName)
	local maxVoltage = -1
	cellId = getTelemetryId(telemetryObjectName)
	if cellId == -1 then
		-- telemetry object not found
		print("telemetry object not found")
	else
		-- check we have table
		cellResult = getValue(cellId)
		if (type(cellResult) == "table") then
			--print("its a table")
			--Value = ""
			-- index, value
			--a = {4.1, 4.0, 4.2,3.99}
			for i, voltage in ipairs(cellResult) do
			--for i, voltage in ipairs(a) do
				
				if maxVoltage < voltage then
					maxVoltage = voltage
					--print("maxVoltage = " .. maxVoltage)
	
				else
					
				end 
			end
		else

		end
	end		
	
	return maxVoltage 
end


local function calcDeltaVolts(maxCellVolts,minCellVolts)
	-- I think the pilot would prefer to see Delta as a voltage (not as percentage)
				
	local deltaVolts	= 0	

		
	-- Trap division by zero
	if 	minCellVolts == 0 then
		minCellVolts = .01
	end
	if 	maxCellVolts == 0 then
		maxCellVolts = .01
	end
				
	-- trap for a bad lipo balance
	deltaVolts =  maxCellVolts - minCellVolts 
	--deltaVolts = myRound(deltaVolts,2)
	--print("deltaVolts in calcDeltaVolts = ",deltaVolts)
	-- eg:- 4.14v (95%) - 3.72v (20%) = .36v would indicate to me faulty lipo
	--		or if using two lipos that one of the lipos has been used!
	-- it is for the pilot to decide what level of Delta should activate a warning.	

	return  deltaVolts
end

local function rssiOK()
local RSSI_ok = false

	-- if getValue("RSSI") > RSSIThreshold
	if getValue("RSSI") > 0 -- keep it simple
			then
				RSSI_ok = true
			else
				RSSI_ok = false
				print("RSSI to low!")
	end			
	return RSSI_ok
end

-- could do with a reset function for when user set throttle hold back on
local function reset()
	-- turn off logging - I do not know how to do this in code	
	
end

local function checkIfCapacityHasChanged()
	local Capacity_mAh = model.getGlobalVariable(GVBatCap, GVFlightMode) * 100
	-- BatCapmAh = LipoCapacity_mAh
	
	return Capacity_mAh
end

local function hasCapacityChanged()
-- Check if battery capacity was changed

-- The user has to stop using the plane by using the exit button
--   and then select menu, then loads model edit
--	   this will trigger the init_func and thus get us here.

	if LipoCapacity_mAh ~= checkIfCapacityHasChanged() 
		then
			-- check for zero value / user has not set a value, 
			--		so just use the hard coded value at the top of this script.
			
			if checkIfCapacityHasChanged() == 0
				then
					-- keep hard coded value
				else
					LipoCapacity_mAh = checkIfCapacityHasChanged()				
			end
		
	end	
	--lipoVoltage = getLipoVoltage("sCelsSensor")
end

-- we could do with a isLipoConnected 
--  because just because they have setup the user created Cel2
--		 and the default Cels sensor exists, that does not mean we have data.
local function isLipoConnected(telemetryObjectName)
	local lipoConnected = false
		lipoConnected = doesSensorExistAndHaveData(telemetryObjectName)

	return lipoConnected
end

local function doesSensorExistAndHaveData(telemetryObjectName)
-- it seems that a sensor can exist but it does not contain data
-- so we need to trap for the existence of a table
	local haveData = false
	cellId = getTelemetryId(telemetryObjectName)
	if cellId == -1 then
		print(telemetryObjectName, "not found")	
	else
		-- check we have table
		cellResult = getValue(cellId)
		if (type(cellResult) == "table") then
			haveData = true
		else
			print(telemetryObjectName, "no table found")
			
		end
	end	

		return haveData
end

local function doesSensorExist(tSensor)
	-- used in the init function

	-- returns true or false
		
	local myReturn = false
	local fieldInfo = ""

		fieldInfo = getFieldInfo(tSensor)

		if fieldInfo
		then
			-- sensor exists
			myReturn = true
		end
	
		return myReturn

end


local function returnMissingSensor(tSensor)
	-- used in the init function

	-- used for trawling an array of sensors or just look for one.
	
	-- this function returns the name of a sensor it cannot find
	-- in other words it returns a parameter with the sensor name it cannot find.
	
	
	local returnString = ""
	local fieldInfo = ""

		fieldInfo = getFieldInfo(tSensor)

		if fieldInfo
		then
			
		else
			-- missing
			returnString = tSensor
			--print ("returnMissingSensor:-Could not find ", tSensor)
		end
	
		return returnString
end


local function trapForMissingSensors()
	-- move sensor checks from init_func to make code more readable
	
	-------------------------------------------
	-- we should trap for missing sensors
	--------------------------------------------
	
	
	--print("trap for missing sensors")
	
	local tempString = ""
	for i, testString in ipairs( myArraySensors ) 
	do
		--print("testString = ",testString)
		missingSensor = returnMissingSensor(testString)
		if missingSensor == "" then
			-- do nothing
		else
			myArrayMissing [#myArrayMissing + 1] = tostring(missingSensor)
		end
	end
	
	missingSensorsCount = CountElements(myArrayMissing)
	--print("missingSensorsCount = ", missingSensorsCount)
	if missingSensorsCount > 0 then
		if missingSensorsCount == 1 then
			missingSensorsWarning = "This sensor must exist"
		else
			-- more than one
			missingSensorsWarning = "These sensors must exist"
		
		end
		
		tempString = soundDirString .. "MisSenso.wav"
		playFile(tempString) -- no trap for missing wav files. could be done
		
	else
		

		missingSensorsWarning = ""

	end

	-------------------------------------------------
	-------------------------------------------------
	
	-------------------------------------------
	-- test for extra sensor e.g. another voltage sensor for a second lipo
	--------------------------------------------
		
		if doesSensorExist("Cel2")  then
			haveWeTwoVoltageSensors = true
			-- also have now way of testing if we should have two sensors or just one ;(
			
			-- now that is all we need to know because we have done away with Max,Min and Delta
				
		else
			-- we only have one voltage sensor Cels
			haveWeTwoVoltageSensors = false
					
		end
	-------------------------------------------------
	-------------------------------------------------
	

end



local function init_func()
-- init_func is called once when model is loaded

	if correctVersionOpenTX() then
	
	lCorrectVersionOpenTX = true
	hasCapacityChanged() -- check battery capacity
	

	-------------------------------------------
	-- we should trap for missing sensors
	--------------------------------------------
	missingSensorsWarning = ""
	trapForMissingSensors()
	
	
	
		if SwitchF ~= "sf"
		then 
			lWeAreUsingSwitchF = false
		else
			lWeAreUsingSwitchF = true
			IdPlayOnDemandSwitch = getFieldInfo(SwitchF).id  
			isFswitchUp = GetIsFswitchUp()
		end
	
	else
		-- do not run anything
	end  -- logicCorrectVersionOpenTX

end


 local function bckgrnd_func()
 -- bckgrnd_func is called periodically when screen is not visible
 
 
local fileNameAndDir = ""

local tmpMaxCellVolts = 0
local tmpMinCellVolts = 0
 
 
 -- because correctVersionOpenTX function is called many times
 --    this function should not be called; instead use variable
 
	 if lCorrectVersionOpenTX and rssiOK() then
	 
			
			-- ThrottleOnOff -1024 = Throttle hold
			-- ThrottleOnOff is the SD switch that has three positions so anything smaller than Zero is throttle hold
			-- Note the SD switch is away from pilot (or up) when throttle hold is true
			--    default position for all switches on my TX are up on front facing switches 
			--			and away from pilot on top switches.
			if getValue(IdSwitch_D) < 0
				then
						logicThrottleHold = true
						-- Note the SD switch is away from pilot when throttle hold is true
		
				else
						logicThrottleHold = false
			end
		
				
		-- Skip this lot if the user has not created the sensors!
		if missingSensorsWarning ~= "" then -- missingSensorsWarning is a string of errors
			-- do nothing
			
		else


			if haveWeTwoVoltageSensors
				then
				--print("We have two voltage sensors")
				
				minCellVolts1 = findMinValue("Cels")
				maxCellVolts1 = findMaxValue("Cels")
				minCellVolts2 = findMinValue("Cel2")
				maxCellVolts2 = findMaxValue("Cel2")
				
				--[[
				-----------------------------------------------------------------
				-- Debugging only
				-----------------------------------------------------------------
				
				-- This is for when we are debugging using the opentx simulator
				-- trap for empty Simulator values
				minCellVolts2 = myRound(getValue(sCMinVotsSensor2),2)
				if minCellVolts2 == 0 then
					--minCellVolts2 = minCellVolts1 - .01
					minCellVolts2 = 4.2
				end
								
				-- trap for empty Simulator values
				maxCellVolts2 = myRound(getValue(sCMaxVotsSensor2),2)
				if maxCellVolts2 == 0 then
					--maxCellVolts2 = maxCellVolts1 - .01
					maxCellVolts2 = 4.2
				end
				
				-----------------------------------------------------------------
				-- above debugging only
				-----------------------------------------------------------------
				]]--
				
				
				tmpMaxCellVolts = higherOfTwo(maxCellVolts1,maxCellVolts2)
				tmpMinCellVolts = lowerOfTwo(minCellVolts1,minCellVolts2)

				-- simple delta voltage
				deltaVolts= calcDeltaVolts(tmpMaxCellVolts,tmpMinCellVolts)

				deltaVolts= myRound(deltaVolts,2)

				
				
					minCommonCellVolts = tmpMinCellVolts
				
				else
					
					minCellVolts1 = findMinValue("Cels")
					maxCellVolts1 = findMaxValue("Cels")

					deltaVolts= calcDeltaVolts(maxCellVolts1,minCellVolts1)
	
					deltaVolts= myRound(deltaVolts,2)
	
					minCommonCellVolts = minCellVolts1
				
			end
			
				
				-- I would rather use a case statement here. 
				-- It seems I could use elseif,elseif to create a case statement

				if minCommonCellVolts > 0 and logicThrottleHold
					then
						reset() -- check battery capacity, as the pilot my have realized the capacity is wrong
					
						-- We are not measuring the current used at this point.
						-- We are giving the pilot an idea has to the state/condition of his/her lipo!
						
						-- Throttle hold is on (the default) and we have telemetry so can read the voltage of the lowest cell
						-- get the resting lipo voltage

						-- look up mAh of battery at the time of installation. (usually fully charged)

						
						-- findPercentRemaining function returns Max 100% and Min of 10% using the packs lowest cell voltage
						estimatedPercentRem = findPercentRemaining(myArrayPercentList,minCommonCellVolts)
						

						lipo_mAh_AtFlightStart = (estimatedPercentRem/100) * LipoCapacity_mAh
							-- show something on the gauge before the Throttle is released
						
										
		
				end 
				
				if minCommonCellVolts > 0 and not logicThrottleHold then
			
						-- we have started flying because throttle has been released and the plane is not to close to TX
						-- we assume!
						
						-- we are now receiving current data!
						mAhs_Consumed = getValue(sMillAmpHourSensor)
						
						lipo_mAh_Remaining = lipo_mAh_AtFlightStart - mAhs_Consumed 
			
						-- lipo_mAh_Remaining express that as a percentage of a full lipo for the Lipo Gauge
						-- the result of the expression below produces massive floating point numbers
						-- Lua does not have an INT function so try math.floor

						calcPercentRem = (lipo_mAh_Remaining/LipoCapacity_mAh) * 100
						-- ceiling crashes 
						-- Do'h it is ceil not ceiling
						
						
						-- It would be better if we knew if we have a sensor or not.
						-- or we just trap for zero volts = lipo not connected.
						
						-- if less than 5% set a trap at 5)
						if  calcPercentRem < 5
							then
								estimatedPercentRem = 5
							else
								estimatedPercentRem = math.ceil(calcPercentRem)
						end		
						selectAndPlaySound(false)
		
				end
							
				-- the telemetry log shows many values unfortunately many are not what the RX sees,
				-- in other words the telemetry system is not a aircraft black box recording every thing in the RX
				-- in opentx 2.0 the all possible values were recorded many of them where just what the TX,
				-- could see being sent to the RX. The TX knowing about what is sent may or my not be received.
				-- In opentx 2.1.9 only values selected by the pilot are added to the log file.
				-- A1 the RX power supply is always sent back to the Tx
				-- as is RSSI; however as stated above the pilot must select the option to record A1 and RSSI.
				
			
				PlaySoundFileOnDemand()
				
		end
	 else
		-- do nothing 
	 end  -- logicCorrectVersionOpenTX
 end

 local function run_func()
  -- run_func is called periodically when screen is visible
  
  

  --print("logicCorrectVersionOpenTX ",logicCorrectVersionOpenTX)
  if lCorrectVersionOpenTX then
	  if missingSensorsWarning == ""
		then
		bckgrnd_func() -- call the background stuff
		else
			-- do nothing
	  end

  
  
		if missingSensorsWarning == ""  then
					lcd.clear() -- clear the screen
					-- show stuff because the screen is active
					x = 33
					y = 1
					lcd.drawText(x, y, myTitle, SMLSIZE+INVERS )
						
					---------------------------------
					-- show the TX gauge
					---------------------------------
					moveDownNLines = 10
					  -- column = x ,row = y
					  -- top,left (0,0)
					lcd.drawPoint(0,0) -- just a dot on the edge of the screen
	
				   lcd.drawText(96,5 + moveDownNLines,"TX Volts = ",SMLSIZE)
				   

					-- getLastPos only seems to give the end of the main string not the total length
				  -- hence the need for + 12 to go more to the right
				  TX_Voltage = getValue(TX_VoltageID) -- actual TX battery voltage
				 
				   lcd.drawNumber(lcd.getLastPos() + 1,5 + moveDownNLines,TX_Voltage * 10 ,PREC1+LEFT+SMLSIZE)

				  
				  
				  txPercent = ((TX_Voltage - txMinVolts) * 100) / ( txMaxVolts - txMinVolts)
				  txPercent = math.ceil(txPercent) -- round up
				 
				   -- it is difficult to get an accurate voltage to what equals 100% so I am going to trap for over 100%
				  -- since a value over 100% is confusing and meaningless.
				  if txPercent > 100 then
						txPercent = 100
				  end
				  -- trap for silly low values when debugging
				  if txPercent < 5 then
						txPercent = 5
				  end
				  
					
				  -- Draw Gauge for TX battery 
				   lcd.drawNumber(178, 5 + moveDownNLines, txPercent, LEFT+SMLSIZE)
				  lcd.drawText(lcd.getLastPos(), 5 + moveDownNLines, "%", SMLSIZE)

				  lcd.drawGauge(157, 3 + moveDownNLines, 50, 10, txPercent, 100)
				  
					-- x,y (positive numbers) top left corner position
					-- w (number) width in pixels
					--- h (number) height in pixels
				lcd.drawRectangle(90, 8, 122, 20, SOLID)

				  
				  
				if rssiOK() then 
				  
					---------------------------------
					-- show the Lipo gauge only if the Lipo is connected
					---------------------------------
					moveDownNLines = 35
					if minCommonCellVolts > 0 then
						  -- column,row
						  -- top,left (0.0)
						  -- bottom, right (63,212)?
							  
						  lcd.drawText(5,4 + moveDownNLines,"Min Cell Voltage = ",0)
						  lcd.drawText(lcd.getLastPos() ,4 + moveDownNLines,tostring(minCommonCellVolts),0)
								 
						  lcd.drawText(6,14+ moveDownNLines,"       Delta volts = ",0)
						  lcd.drawText(lcd.getLastPos() ,14 + moveDownNLines ,tostring(deltaVolts),0)
						  

						   -- Draw Gauge for Lipo battery
						  lcd.drawNumber(153, 6 + moveDownNLines, estimatedPercentRem, LEFT+MIDSIZE)
						  lcd.drawText(lcd.getLastPos(), 6 + moveDownNLines, "%", MIDSIZE)
						  lcd.drawGauge(120, 3 + moveDownNLines, 88, 18, estimatedPercentRem, 100)
						  
						else
							-- no voltage
					
							lcd.drawText(20,9 + moveDownNLines,"Lipo NOT connected!",0)
						
					end
			
					lcd.drawText(12,22,"Lipo " .. tostring(LipoCapacity_mAh) .. " mAh",	SMLSIZE)
 
				end
			
		else
			-- do nothing
		end	-- missingSensorsWarning

			-- use one missingSensorsWarning string for normal sensors and extra sensors
			-- print the array so we can see it

		
		if missingSensorsCount > 0 then
			-- clear the screen because we must sort this out first
			lcd.clear()
			-- we have something to say about sensors

				
				-- column = x ,row = y
				-- top,left (0,0)
				local row = 1
				local moveDown = 8
				local line1 = "After correcting the above"
				local line2 = "you must force this script to reload"
				local line3 = "by selecting another model"
				local line4 = "then reselect this model"
				local line5 = "OR "
				local line6 = "Just turn transmitter off and back on again"
				
				
				centreText(missingSensorsWarning,row)
				row = row + moveDown
				listOfMissingSensors = ConcatString(myArrayMissing)
				centreText(listOfMissingSensors,row)
				
				row = row + moveDown
				centreText(line1,row)
				row = row + moveDown
				centreText(line2,row)
				row = row + moveDown
				centreText(line3,row)				
				row = row + moveDown
				centreText(line4,row)
				row = row + moveDown
				centreText(line5,row)
				row = row + moveDown
				centreText(line6,row)
		
		else	
			

			
			
		end	 -- howManyMissingSensors
	else 
		-- warn user wrong version
		lcd.clear()
		lcd.drawText(10, 15, "This script/program only", MIDSIZE )
		lcd.drawText(10, 35, " tested with OpenTX 2.1.9", MIDSIZE )
	end -- logicCorrectVersionOpenTX
end
  return { run=run_func,  background=bckgrnd_func ,init=init_func }


